@@ -102,9 +102,9 @@ enum
 
 static GParamSpec *obj_props[PROP_LAST];
 
-G_DEFINE_ABSTRACT_TYPE (ClutterDeformEffect,
-                        clutter_deform_effect,
-                        CLUTTER_TYPE_OFFSCREEN_EFFECT);
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (ClutterDeformEffect,
+                                     clutter_deform_effect,
+                                     CLUTTER_TYPE_OFFSCREEN_EFFECT)
 
 static void
 clutter_deform_effect_real_deform_vertex (ClutterDeformEffect *effect,
@@ -180,6 +180,7 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
 
   if (priv->is_dirty)
     {
+      ClutterRect rect;
       gboolean mapped_buffer;
       CoglVertexP3T2C4 *verts;
       ClutterActor *actor;
@@ -193,7 +194,12 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
       /* if we don't have a target size, fall back to the actor's
        * allocation, though wrong it might be
        */
-      if (!clutter_offscreen_effect_get_target_size (effect, &width, &height))
+      if (clutter_offscreen_effect_get_target_rect (effect, &rect))
+        {
+          width = clutter_rect_get_width (&rect);
+          height = clutter_rect_get_height (&rect);
+        }
+      else
         clutter_actor_get_size (actor, &width, &height);
 
       /* XXX ideally, the sub-classes should tell us what they
@@ -571,8 +577,6 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   ClutterOffscreenEffectClass *offscreen_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterDeformEffectPrivate));
-
   klass->deform_vertex = clutter_deform_effect_real_deform_vertex;
 
   /**
@@ -639,9 +643,7 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
 static void
 clutter_deform_effect_init (ClutterDeformEffect *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_DEFORM_EFFECT,
-                                            ClutterDeformEffectPrivate);
-
+  self->priv = clutter_deform_effect_get_instance_private (self);
   self->priv->x_tiles = self->priv->y_tiles = DEFAULT_N_TILES;
   self->priv->back_pipeline = NULL;
 
