@@ -150,7 +150,7 @@ static gboolean on_captured_event (ClutterActor      *stage,
                                    ClutterEvent      *event,
                                    ClutterDragAction *action);
 
-G_DEFINE_TYPE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION);
+G_DEFINE_TYPE_WITH_PRIVATE (ClutterDragAction, clutter_drag_action, CLUTTER_TYPE_ACTION)
 
 static void
 get_drag_threshold (ClutterDragAction *action,
@@ -364,14 +364,14 @@ on_captured_event (ClutterActor      *stage,
   if (!priv->in_drag)
     return CLUTTER_EVENT_PROPAGATE;
 
-  if (clutter_event_get_device (event) != priv->device ||
-      clutter_event_get_event_sequence (event) != priv->sequence)
+  if (clutter_event_get_device (event) != priv->device)
     return CLUTTER_EVENT_PROPAGATE;
 
   switch (clutter_event_type (event))
     {
     case CLUTTER_TOUCH_UPDATE:
-      emit_drag_motion (action, actor, event);
+      if (clutter_event_get_event_sequence (event) == priv->sequence)
+        emit_drag_motion (action, actor, event);
       break;
 
     case CLUTTER_MOTION:
@@ -391,7 +391,8 @@ on_captured_event (ClutterActor      *stage,
 
     case CLUTTER_TOUCH_END:
     case CLUTTER_TOUCH_CANCEL:
-      emit_drag_end (action, actor, event);
+      if (clutter_event_get_event_sequence (event) == priv->sequence)
+        emit_drag_end (action, actor, event);
       break;
 
     case CLUTTER_BUTTON_RELEASE:
@@ -426,8 +427,6 @@ on_drag_begin (ClutterActor      *actor,
   switch (clutter_event_type (event))
     {
     case CLUTTER_BUTTON_PRESS:
-      if (priv->sequence != NULL)
-        return CLUTTER_EVENT_PROPAGATE;
       if (clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
         return CLUTTER_EVENT_PROPAGATE;
       break;
@@ -711,8 +710,6 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (ClutterDragActionPrivate));
-
   meta_class->set_actor = clutter_drag_action_set_actor;
 
   klass->drag_progress = clutter_drag_action_real_drag_progress;
@@ -992,8 +989,7 @@ clutter_drag_action_class_init (ClutterDragActionClass *klass)
 static void
 clutter_drag_action_init (ClutterDragAction *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_DRAG_ACTION,
-                                            ClutterDragActionPrivate);
+  self->priv = clutter_drag_action_get_instance_private (self);
 }
 
 /**
@@ -1142,18 +1138,14 @@ clutter_drag_action_set_drag_handle (ClutterDragAction *action,
 
   priv->transformed_press_x = priv->press_x;
   priv->transformed_press_y = priv->press_y;
+  clutter_actor_transform_stage_point (handle, priv->press_x, priv->press_y,
+                                       &priv->transformed_press_x,
+                                       &priv->transformed_press_y);
 
   if (priv->drag_handle != NULL)
-    {
-      clutter_actor_transform_stage_point (priv->drag_handle,
-                                           priv->press_x,
-                                           priv->press_y,
-					   &priv->transformed_press_x,
-					   &priv->transformed_press_y);
-      g_signal_connect (priv->drag_handle, "destroy",
-			G_CALLBACK (on_drag_handle_destroy),
-			action);
-    }
+    g_signal_connect (priv->drag_handle, "destroy",
+                      G_CALLBACK (on_drag_handle_destroy),
+                      action);
 
   g_object_notify_by_pspec (G_OBJECT (action), drag_props[PROP_DRAG_HANDLE]);
 }
