@@ -46,8 +46,6 @@
 #include "clutter-private.h"
 #include "clutter-stage-private.h"
 
-#include <math.h>
-
 enum
 {
   PROP_0,
@@ -1201,9 +1199,6 @@ _clutter_input_device_translate_axis (ClutterInputDevice *device,
       info->axis == CLUTTER_INPUT_AXIS_Y)
     return FALSE;
 
-  if (fabs (info->max_value - info->min_value) < 0.0000001)
-    return FALSE;
-
   width = info->max_value - info->min_value;
   real_value = (info->max_axis * (value - info->min_value)
              + info->min_axis * (info->max_value - value))
@@ -1620,28 +1615,6 @@ clutter_input_device_get_associated_device (ClutterInputDevice *device)
   return device->associated;
 }
 
-/*< internal >
- * clutter_input_device_select_stage_events:
- * @device: a #ClutterInputDevice
- * @stage: the #ClutterStage to select events on
- * @event_mask: platform-specific mask of events
- *
- * Selects input device events on @stage.
- *
- * The implementation of this function depends on the backend used.
- */
-void
-_clutter_input_device_select_stage_events (ClutterInputDevice *device,
-                                           ClutterStage       *stage,
-                                           gint                event_mask)
-{
-  ClutterInputDeviceClass *device_class;
-
-  device_class = CLUTTER_INPUT_DEVICE_GET_CLASS (device);
-  if (device_class->select_stage_events != NULL)
-    device_class->select_stage_events (device, stage, event_mask);
-}
-
 /**
  * clutter_input_device_keycode_to_evdev:
  * @device: A #ClutterInputDevice
