@@ -51,6 +51,15 @@
 
 #include "clutter-device-manager-evdev.h"
 
+/* These are the same as the XInput2 IDs */
+#define CORE_POINTER_ID 2
+#define CORE_KEYBOARD_ID 3
+
+#define FIRST_SLAVE_ID 4
+#define INVALID_SLAVE_ID 255
+
+#define AUTOREPEAT_VALUE 2
+
 struct _ClutterDeviceManagerEvdevPrivate
 {
   GUdevClient *udev_client;
@@ -67,12 +76,19 @@ struct _ClutterDeviceManagerEvdevPrivate
   ClutterStageManager *stage_manager;
   guint stage_added_handler;
   guint stage_removed_handler;
+
+  GArray *keys;
+  struct xkb_state *xkb;              /* XKB state object */
+  uint32_t button_state;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (ClutterDeviceManagerEvdev,
                             clutter_device_manager_evdev,
                             CLUTTER_TYPE_DEVICE_MANAGER)
 
+static ClutterOpenDeviceCallback open_callback;
+static gpointer                  open_callback_data;
+
 static const gchar *subsystems[] = { "input", NULL };
 
 /*
@@ -102,11 +118,8 @@ struct _ClutterEventSource
 {
   GSource source;
 
-  ClutterInputDeviceEvdev *device;    /* back pointer to the evdev device */
+  ClutterInputDeviceEvdev *device;    /* back pointer to the slave evdev device */
   GPollFD event_poll_fd;              /* file descriptor of the /dev node */
-  struct xkb_state *xkb;              /* XKB state object */
-  gint x, y;                          /* last x, y position for pointers */
-  guint32 modifier_state;             /* key modifiers */
 };
 
 static gboolean
@@ -144,12 +157,32 @@ clutter_event_check (GSource *source)
 static void
 queue_event (ClutterEvent *event)
 {
-  if (event == NULL)
-    return;
-
   _clutter_event_push (event, FALSE);
 }
 
+static void
+add_key (GArray  *keys,
+	 guint32  key)
+{
+  g_array_append_val (keys, key);
+}
+
+static void
+remove_key (GArray  *keys,
+	    guint32  key)
+{
+  unsigned int i;
+
+  for (i = 0; i < keys->len; i++)
+    {
+      if (g_array_index (keys, guint32, i) == key)
+	{
+	  g_array_remove_index_fast (keys, i);
+	  return;
+	}
+    }
+}
+
 static void
 notify_key (ClutterEventSource *source,
             guint32             time_,
@@ -157,6 +190,7 @@ notify_key (ClutterEventSource *source,
             guint32             state)
 {
   ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
+  ClutterDeviceManagerEvdev *manager_evdev;
   ClutterStage *stage;
   ClutterEvent *event = NULL;
 
@@ -166,31 +200,48 @@ notify_key (ClutterEventSource *source,
   if (!stage)
     return;
 
-  /* if we have a mapping for that device, use it to generate the event */
-  if (source->xkb)
-  {
-    event =
-      _clutter_key_event_new_from_evdev (input_device,
-                                         stage,
-                                         source->xkb,
-                                         time_, key, state);
-    xkb_state_update_key (source->xkb, key, state ? XKB_KEY_DOWN : XKB_KEY_UP);
-  }
+  manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (input_device->device_manager);
 
-  queue_event (event);
+  /* if we have keyboard state, use it to generate the event */
+  if (manager_evdev->priv->xkb)
+    {
+      event =
+	_clutter_key_event_new_from_evdev (input_device,
+					   manager_evdev->priv->core_keyboard,
+					   stage,
+					   manager_evdev->priv->xkb,
+					   manager_evdev->priv->button_state,
+					   time_, key, state);
+
+      /* We must be careful and not pass multiple releases to xkb, otherwise it gets
+	 confused and locks the modifiers */
+      if (state != AUTOREPEAT_VALUE)
+	{
+	  xkb_state_update_key (manager_evdev->priv->xkb, event->key.hardware_keycode, state ? XKB_KEY_DOWN : XKB_KEY_UP);
+
+	  if (state)
+	    add_key (manager_evdev->priv->keys, event->key.hardware_keycode);
+	  else
+	    remove_key (manager_evdev->priv->keys, event->key.hardware_keycode);
+	}
+
+      queue_event (event);
+    }
 }
 
 
 static void
-notify_motion (ClutterEventSource *source,
-               guint32             time_,
-               gint                x,
-               gint                y)
+notify_relative_motion (ClutterEventSource *source,
+			guint32             time_,
+			gint                dx,
+			gint                dy)
 {
   ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
   gfloat stage_width, stage_height, new_x, new_y;
-  ClutterEvent *event;
+  ClutterDeviceManagerEvdev *manager_evdev;
   ClutterStage *stage;
+  ClutterEvent *event = NULL;
+  ClutterPoint point;
 
   /* We can drop the event on the floor if no stage has been
    * associated with the device yet. */
@@ -198,34 +249,62 @@ notify_motion (ClutterEventSource *source,
   if (!stage)
     return;
 
+  manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (input_device->device_manager);
+
   stage_width = clutter_actor_get_width (CLUTTER_ACTOR (stage));
   stage_height = clutter_actor_get_height (CLUTTER_ACTOR (stage));
 
   event = clutter_event_new (CLUTTER_MOTION);
 
-  if (x < 0)
-    new_x = 0.f;
-  else if (x >= stage_width)
-    new_x = stage_width - 1;
-  else
-    new_x = x;
-
-  if (y < 0)
-    new_y = 0.f;
-  else if (y >= stage_height)
-    new_y = stage_height - 1;
-  else
-    new_y = y;
-
-  source->x = new_x;
-  source->y = new_y;
+  clutter_input_device_get_coords (manager_evdev->priv->core_pointer, NULL, &point);
+  new_x = point.x + dx;
+  new_y = point.y + dy;
+  new_x = CLAMP (new_x, 0.f, stage_width - 1);
+  new_y = CLAMP (new_y, 0.f, stage_height - 1);
 
   event->motion.time = time_;
   event->motion.stage = stage;
-  event->motion.device = input_device;
-  event->motion.modifier_state = source->modifier_state;
+  event->motion.device = manager_evdev->priv->core_pointer;
+  event->motion.modifier_state = xkb_state_serialize_mods (manager_evdev->priv->xkb, XKB_STATE_EFFECTIVE);
+  event->motion.modifier_state |= manager_evdev->priv->button_state;
   event->motion.x = new_x;
   event->motion.y = new_y;
+  clutter_event_set_source_device (event, input_device);
+
+  queue_event (event);
+}
+
+static void
+notify_scroll (ClutterEventSource *source,
+	       guint32             time_,
+	       gint32              value)
+{
+  ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
+  ClutterDeviceManagerEvdev *manager_evdev;
+  ClutterStage *stage;
+  ClutterEvent *event = NULL;
+  ClutterPoint point;
+
+  /* We can drop the event on the floor if no stage has been
+   * associated with the device yet. */
+  stage = _clutter_input_device_get_stage (input_device);
+  if (!stage)
+    return;
+
+  manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (input_device->device_manager);
+
+  event = clutter_event_new (CLUTTER_SCROLL);
+
+  event->scroll.time = time_;
+  event->scroll.stage = CLUTTER_STAGE (stage);
+  event->scroll.device = manager_evdev->priv->core_pointer;
+  event->motion.modifier_state = xkb_state_serialize_mods (manager_evdev->priv->xkb, XKB_STATE_EFFECTIVE);
+  event->scroll.modifier_state |= manager_evdev->priv->button_state;
+  event->scroll.direction = value < 0 ? CLUTTER_SCROLL_DOWN : CLUTTER_SCROLL_UP;
+  clutter_input_device_get_coords (manager_evdev->priv->core_pointer, NULL, &point);
+  event->scroll.x = point.x;
+  event->scroll.y = point.y;
+  clutter_event_set_source_device (event, (ClutterInputDevice*) source->device);
 
   queue_event (event);
 }
@@ -237,8 +316,10 @@ notify_button (ClutterEventSource *source,
                guint32             state)
 {
   ClutterInputDevice *input_device = (ClutterInputDevice *) source->device;
-  ClutterEvent *event;
+  ClutterDeviceManagerEvdev *manager_evdev;
   ClutterStage *stage;
+  ClutterEvent *event = NULL;
+  ClutterPoint point;
   gint button_nr;
   static gint maskmap[8] =
     {
@@ -252,6 +333,8 @@ notify_button (ClutterEventSource *source,
   if (!stage)
     return;
 
+  manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (input_device->device_manager);
+
   /* The evdev button numbers don't map sequentially to clutter button
    * numbers (the right and middle mouse buttons are in the opposite
    * order) so we'll map them directly with a switch statement */
@@ -287,17 +370,20 @@ notify_button (ClutterEventSource *source,
 
   /* Update the modifiers */
   if (state)
-    source->modifier_state |= maskmap[button - BTN_LEFT];
+    manager_evdev->priv->button_state |= maskmap[button - BTN_LEFT];
   else
-    source->modifier_state &= ~maskmap[button - BTN_LEFT];
+    manager_evdev->priv->button_state &= ~maskmap[button - BTN_LEFT];
 
   event->button.time = time_;
   event->button.stage = CLUTTER_STAGE (stage);
-  event->button.device = (ClutterInputDevice *) source->device;
-  event->button.modifier_state = source->modifier_state;
+  event->button.device = manager_evdev->priv->core_pointer;
+  event->motion.modifier_state = xkb_state_serialize_mods (manager_evdev->priv->xkb, XKB_STATE_EFFECTIVE);
+  event->button.modifier_state |= manager_evdev->priv->button_state;
   event->button.button = button_nr;
-  event->button.x = source->x;
-  event->button.y = source->y;
+  clutter_input_device_get_coords (manager_evdev->priv->core_pointer, NULL, &point);
+  event->button.x = point.x;
+  event->button.y = point.y;
+  clutter_event_set_source_device (event, (ClutterInputDevice*) source->device);
 
   queue_event (event);
 }
@@ -360,7 +446,6 @@ clutter_event_dispatch (GSource     *g_source,
            struct input_event *e = &ev[i];
 
            _time = e->time.tv_sec * 1000 + e->time.tv_usec / 1000;
-           event = NULL;
 
            switch (e->type)
              {
@@ -368,7 +453,7 @@ clutter_event_dispatch (GSource     *g_source,
 
                /* don't repeat mouse buttons */
                if (e->code >= BTN_MOUSE && e->code < KEY_OK)
-                 if (e->value == 2)
+                 if (e->value == AUTOREPEAT_VALUE)
                    continue;
 
                switch (e->code)
@@ -419,6 +504,14 @@ clutter_event_dispatch (GSource     *g_source,
                  case REL_Y:
                    dy += e->value;
                    break;
+
+		 /* Note: we assume that REL_WHEEL is for *vertical* scroll wheels.
+		    To implement horizontal scroll, we'll need a different enum
+		    value.
+		 */
+		 case REL_WHEEL:
+		   notify_scroll (source, _time, e->value);
+		   break;
                  }
                break;
 
@@ -427,12 +520,10 @@ clutter_event_dispatch (GSource     *g_source,
                g_warning ("Unhandled event of type %d", e->type);
                break;
              }
-
-           queue_event (event);
          }
 
        if (dx != 0 || dy != 0)
-         notify_motion (source, _time, source->x + dx, source->y + dy);
+         notify_relative_motion (source, _time, dx, dy);
     }
 
   /* Pop an event off the queue if any */
@@ -462,20 +553,35 @@ clutter_event_source_new (ClutterInputDeviceEvdev *input_device)
 {
   GSource *source = g_source_new (&event_funcs, sizeof (ClutterEventSource));
   ClutterEventSource *event_source = (ClutterEventSource *) source;
-  ClutterInputDeviceType type;
   const gchar *node_path;
   gint fd;
+  GError *error;
 
   /* grab the udev input device node and open it */
   node_path = _clutter_input_device_evdev_get_device_path (input_device);
 
   CLUTTER_NOTE (EVENT, "Creating GSource for device %s", node_path);
 
-  fd = open (node_path, O_RDONLY | O_NONBLOCK);
-  if (fd < 0)
+  if (open_callback)
     {
-      g_warning ("Could not open device %s: %s", node_path, strerror (errno));
-      return NULL;
+      error = NULL;
+      fd = open_callback (node_path, O_RDONLY | O_NONBLOCK, open_callback_data, &error);
+
+      if (fd < 0)
+	{
+	  g_warning ("Could not open device %s: %s", node_path, error->message);
+	  g_error_free (error);
+	  return NULL;
+	}
+    }
+  else
+    {
+      fd = open (node_path, O_RDONLY | O_NONBLOCK);
+      if (fd < 0)
+	{
+	  g_warning ("Could not open device %s: %s", node_path, strerror (errno));
+	  return NULL;
+	}
     }
 
   /* setup the source */
@@ -483,31 +589,6 @@ clutter_event_source_new (ClutterInputDeviceEvdev *input_device)
   event_source->event_poll_fd.fd = fd;
   event_source->event_poll_fd.events = G_IO_IN;
 
-  type =
-    clutter_input_device_get_device_type (CLUTTER_INPUT_DEVICE (input_device));
-
-  if (type == CLUTTER_KEYBOARD_DEVICE)
-    {
-      /* create the xkb description */
-      event_source->xkb = _clutter_xkb_state_new (NULL,
-                                                  option_xkb_layout,
-                                                  option_xkb_variant,
-                                                  option_xkb_options);
-      if (G_UNLIKELY (event_source->xkb == NULL))
-        {
-          g_warning ("Could not compile keymap %s:%s:%s", option_xkb_layout,
-                     option_xkb_variant, option_xkb_options);
-          close (fd);
-          g_source_unref (source);
-          return NULL;
-        }
-    }
-  else if (type == CLUTTER_POINTER_DEVICE)
-    {
-      event_source->x = 0;
-      event_source->y = 0;
-    }
-
   /* and finally configure and attach the GSource */
   g_source_set_priority (source, CLUTTER_PRIORITY_EVENTS);
   g_source_add_poll (source, &event_source->event_poll_fd);
@@ -574,6 +655,7 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   ClutterInputDeviceType type = CLUTTER_EXTENSION_DEVICE;
   ClutterInputDevice *device;
   const gchar *device_file, *sysfs_path, *device_name;
+  int id, ok;
 
   device_file = g_udev_device_get_device_file (udev_device);
   sysfs_path = g_udev_device_get_sysfs_path (udev_device);
@@ -606,10 +688,18 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   else if (g_udev_device_has_property (udev_device, "ID_INPUT_TOUCHSCREEN"))
     type = CLUTTER_TOUCHSCREEN_DEVICE;
 
+  ok = sscanf (device_file, "/dev/input/event%d", &id);
+  if (ok == 1)
+    id += FIRST_SLAVE_ID;
+  else
+    id = INVALID_SLAVE_ID;
+
   device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_EVDEV,
-                         "id", 0,
+                         "id", id,
                          "name", device_name,
+			 "device-manager", manager,
                          "device-type", type,
+			 "device-mode", CLUTTER_INPUT_MODE_SLAVE,
                          "sysfs-path", sysfs_path,
                          "device-path", device_file,
                          "enabled", TRUE,
@@ -618,6 +708,16 @@ evdev_add_device (ClutterDeviceManagerEvdev *manager_evdev,
   _clutter_input_device_set_stage (device, manager_evdev->priv->stage);
 
   _clutter_device_manager_add_device (manager, device);
+  if (type == CLUTTER_KEYBOARD_DEVICE)
+    {
+      _clutter_input_device_set_associated_device (device, manager_evdev->priv->core_keyboard);
+      _clutter_input_device_add_slave (manager_evdev->priv->core_keyboard, device);
+    }
+  else
+    {
+      _clutter_input_device_set_associated_device (device, manager_evdev->priv->core_pointer);
+      _clutter_input_device_add_slave (manager_evdev->priv->core_pointer, device);
+    }
 
   CLUTTER_NOTE (EVENT, "Added device %s, type %d, sysfs %s",
                 device_file, type, sysfs_path);
@@ -696,9 +796,7 @@ clutter_device_manager_evdev_add_device (ClutterDeviceManager *manager,
 {
   ClutterDeviceManagerEvdev *manager_evdev;
   ClutterDeviceManagerEvdevPrivate *priv;
-  ClutterInputDeviceType device_type;
   ClutterInputDeviceEvdev *device_evdev;
-  gboolean is_pointer, is_keyboard;
   GSource *source;
 
   manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (manager);
@@ -706,22 +804,16 @@ clutter_device_manager_evdev_add_device (ClutterDeviceManager *manager,
 
   device_evdev = CLUTTER_INPUT_DEVICE_EVDEV (device);
 
-  device_type = clutter_input_device_get_device_type (device);
-  is_pointer  = device_type == CLUTTER_POINTER_DEVICE;
-  is_keyboard = device_type == CLUTTER_KEYBOARD_DEVICE;
-
   priv->devices = g_slist_prepend (priv->devices, device);
 
-  if (is_pointer && priv->core_pointer == NULL)
-    priv->core_pointer = device;
+  if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_SLAVE)
+    {
+      /* Install the GSource for this device */
+      source = clutter_event_source_new (device_evdev);
+      g_assert (source != NULL);
 
-  if (is_keyboard && priv->core_keyboard == NULL)
-    priv->core_keyboard = device;
-
-  /* Install the GSource for this device */
-  source = clutter_event_source_new (device_evdev);
-  if (G_LIKELY (source))
-    priv->event_sources = g_slist_prepend (priv->event_sources, source);
+      priv->event_sources = g_slist_prepend (priv->event_sources, source);
+    }
 }
 
 static void
@@ -738,16 +830,19 @@ clutter_device_manager_evdev_remove_device (ClutterDeviceManager *manager,
   /* Remove the device */
   priv->devices = g_slist_remove (priv->devices, device);
 
-  /* Remove the source */
-  source = find_source_by_device (manager_evdev, device);
-  if (G_UNLIKELY (source == NULL))
+  if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_SLAVE)
     {
-      g_warning ("Trying to remove a device without a source installed ?!");
-      return;
-    }
+      /* Remove the source */
+      source = find_source_by_device (manager_evdev, device);
+      if (G_UNLIKELY (source == NULL))
+	{
+	  g_warning ("Trying to remove a device without a source installed ?!");
+	  return;
+	}
 
-  clutter_event_source_free (source);
-  priv->event_sources = g_slist_remove (priv->event_sources, source);
+      clutter_event_source_free (source);
+      priv->event_sources = g_slist_remove (priv->event_sources, source);
+    }
 }
 
 static const GSList *
@@ -830,10 +925,41 @@ clutter_device_manager_evdev_constructed (GObject *gobject)
 {
   ClutterDeviceManagerEvdev *manager_evdev;
   ClutterDeviceManagerEvdevPrivate *priv;
+  ClutterInputDevice *device;
 
   manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (gobject);
   priv = manager_evdev->priv;
 
+  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_EVDEV,
+                         "id", CORE_POINTER_ID,
+                         "name", "input/core_pointer",
+			 "device-manager", manager_evdev,
+                         "device-type", CLUTTER_POINTER_DEVICE,
+			 "device-mode", CLUTTER_INPUT_MODE_MASTER,
+                         "enabled", TRUE,
+                         NULL);  
+  _clutter_input_device_set_stage (device, priv->stage);
+  _clutter_device_manager_add_device (CLUTTER_DEVICE_MANAGER (manager_evdev), device);
+  priv->core_pointer = device;
+
+  device = g_object_new (CLUTTER_TYPE_INPUT_DEVICE_EVDEV,
+                         "id", CORE_KEYBOARD_ID,
+                         "name", "input/core_keyboard",
+			 "device-manager", manager_evdev,
+                         "device-type", CLUTTER_KEYBOARD_DEVICE,
+			 "device-mode", CLUTTER_INPUT_MODE_MASTER,
+                         "enabled", TRUE,
+                         NULL);
+  _clutter_input_device_set_stage (device, priv->stage);
+  _clutter_device_manager_add_device (CLUTTER_DEVICE_MANAGER (manager_evdev), device);
+  priv->core_keyboard = device;
+
+  priv->keys = g_array_new (FALSE, FALSE, sizeof (guint32));
+  priv->xkb = _clutter_xkb_state_new (NULL,
+				      option_xkb_layout,
+				      option_xkb_variant,
+				      option_xkb_options);
+
   priv->udev_client = g_udev_client_new (subsystems);
 
   clutter_device_manager_evdev_probe_devices (manager_evdev);
@@ -1119,3 +1245,71 @@ clutter_evdev_reclaim_devices (void)
   priv->released = FALSE;
   clutter_device_manager_evdev_probe_devices (evdev_manager);
 }
+
+/**
+ * clutter_evdev_set_open_callback: (skip)
+ * @callback: the user replacement for open()
+ * @user_data: user data for @callback
+ *
+ * Through this function, the application can set a custom callback
+ * to invoked when Clutter is about to open an evdev device. It can do
+ * so if special handling is needed, for example to circumvent permission
+ * problems.
+ *
+ * Setting @callback to %NULL will reset the default behavior.
+ *
+ * For reliable effects, this function must be called before clutter_init().
+ */
+void
+clutter_evdev_set_open_callback (ClutterOpenDeviceCallback callback,
+                                 gpointer                  user_data)
+{
+  open_callback = callback;
+  open_callback_data = user_data;
+}
+
+/**
+ * clutter_evdev_get_keyboard_state: (skip)
+ * @evdev: the #ClutterDeviceManager created by the evdev backend
+ *
+ * Returns the xkb state tracking object for keyboard devices.
+ * The object must be treated as read only, and should be used only
+ * for reading out the detailed group and modifier state.
+ *
+ * Return value: the #xkb_state struct
+ */
+struct xkb_state *
+clutter_evdev_get_keyboard_state (ClutterDeviceManager *evdev)
+{
+  g_return_val_if_fail (CLUTTER_IS_DEVICE_MANAGER_EVDEV (evdev), NULL);
+
+  return (CLUTTER_DEVICE_MANAGER_EVDEV (evdev))->priv->xkb;
+}
+
+/**
+ * clutter_evdev_set_keyboard_map: (skip)
+ * @evdev: the #ClutterDeviceManager created by the evdev backend
+ * @keymap: the new keymap
+ *
+ * Instructs @evdev to use the speficied keyboard map. This will cause
+ * the backend to drop the state and create a new one with the new map.
+ */
+void
+clutter_evdev_set_keyboard_map (ClutterDeviceManager *evdev,
+				struct xkb_keymap    *keymap)
+{
+  ClutterDeviceManagerEvdev *manager_evdev;
+  ClutterDeviceManagerEvdevPrivate *priv;
+  unsigned int i;
+
+  g_return_if_fail (CLUTTER_IS_DEVICE_MANAGER_EVDEV (evdev));
+
+  manager_evdev = CLUTTER_DEVICE_MANAGER_EVDEV (evdev);
+  priv = manager_evdev->priv;
+
+  xkb_state_unref (priv->xkb);
+  priv->xkb = xkb_state_new (keymap);
+
+  for (i = 0; i < priv->keys->len; i++)
+    xkb_state_update_key (priv->xkb, g_array_index (priv->keys, guint32, i), XKB_KEY_DOWN);
+}
