@@ -113,15 +113,17 @@ clutter_stage_wayland_realize (ClutterStageWindow *stage_window)
 
   wl_surface = cogl_wayland_onscreen_get_surface (stage_cogl->onscreen);
   wl_surface_set_user_data (wl_surface, stage_wayland);
-
-  wl_shell_surface =
-    cogl_wayland_onscreen_get_shell_surface (stage_cogl->onscreen);
-  wl_shell_surface_add_listener (wl_shell_surface,
-                                 &shell_surface_listener,
-                                 stage_wayland);
-
   stage_wayland->wayland_surface = wl_surface;
-  stage_wayland->wayland_shell_surface = wl_shell_surface;
+
+  if (!stage_wayland->foreign_wl_surface)
+    {
+      wl_shell_surface =
+        cogl_wayland_onscreen_get_shell_surface (stage_cogl->onscreen);
+      wl_shell_surface_add_listener (wl_shell_surface,
+                                     &shell_surface_listener,
+                                     stage_wayland);
+      stage_wayland->wayland_shell_surface = wl_shell_surface;
+    }
 
   if (stage_wayland->fullscreen)
     clutter_stage_wayland_set_fullscreen (stage_window, TRUE);
@@ -138,8 +140,10 @@ clutter_stage_wayland_show (ClutterStageWindow *stage_window,
 
   clutter_stage_window_parent_iface->show (stage_window, do_raise);
 
-  /* TODO: must not call this on foreign surfaces when we add that support */
-  wl_shell_surface_set_toplevel (stage_wayland->wayland_shell_surface);
+  if (stage_wayland->wayland_shell_surface)
+    wl_shell_surface_set_toplevel (stage_wayland->wayland_shell_surface);
+
+  stage_wayland->shown = TRUE;
 
   /* We need to queue a redraw after the stage is shown because all of
    * the other queue redraws up to this point will have been ignored
@@ -161,7 +165,7 @@ clutter_stage_wayland_set_fullscreen (ClutterStageWindow *stage_window,
 
   stage_wayland->fullscreen = fullscreen;
 
-  if (!stage_wayland->wayland_shell_surface) /* Not realized yet */
+  if (!stage_wayland->wayland_shell_surface)
     return;
 
   if (fullscreen)
@@ -203,12 +207,16 @@ clutter_stage_wayland_resize (ClutterStageWindow *stage_window,
                               gint                height)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+  ClutterStageWayland *stage_wayland = CLUTTER_STAGE_WAYLAND (stage_window);
 
   /* Resize preserving top left */
   if (stage_cogl->onscreen)
     {
       cogl_wayland_onscreen_resize (stage_cogl->onscreen, width, height, 0, 0);
-      _clutter_stage_window_redraw (stage_window);
+
+      /* Only trigger a redraw if the stage window has been shown */
+      if (stage_wayland->shown)
+        _clutter_stage_window_redraw (stage_window);
     }
 }
 
@@ -290,3 +298,50 @@ clutter_wayland_stage_get_wl_surface (ClutterStage *stage)
   return stage_wayland->wayland_surface;
 
 }
+
+/**
+ * clutter_wayland_stage_set_wl_surface:
+ * @stage: a #ClutterStage
+ * @surface: A Wayland surface to associate with the @stage.
+ *
+ * Allows you to explicitly provide an existing Wayland surface to associate
+ * with @stage, preventing Cogl from allocating a surface and shell surface for
+ * the stage automatically.
+ *
+ * This function must be called before @stage is shown.
+ *
+ * Note: this function can only be called when running on the Wayland
+ * platform. Calling this function at any other time has no effect.
+ *
+ * Since: 1.16
+ */
+void
+clutter_wayland_stage_set_wl_surface (ClutterStage *stage,
+                                      struct wl_surface *surface)
+{
+  ClutterStageWindow *stage_window = _clutter_stage_get_window (stage);
+  ClutterStageWayland *stage_wayland;
+  ClutterStageCogl *stage_cogl;
+
+  if (!CLUTTER_IS_STAGE_WAYLAND (stage_window))
+    return;
+
+  stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+
+  if (stage_cogl->onscreen == NULL)
+    {
+      ClutterBackend *backend = clutter_get_default_backend ();
+
+      /* Use the same default dimensions as clutter_stage_cogl_realize() */
+      stage_cogl->onscreen = cogl_onscreen_new (backend->cogl_context,
+                                                800, 600);
+
+      cogl_wayland_onscreen_set_foreign_surface (stage_cogl->onscreen,
+                                                 surface);
+
+      stage_wayland = CLUTTER_STAGE_WAYLAND (stage_window);
+      stage_wayland->foreign_wl_surface = TRUE;
+    }
+  else
+    g_warning (G_STRLOC ": cannot set foreign surface for stage");
+}
